function [D, A, B, nFactors] = SMFR_Prox_LinX(X, Y, lam1, lam2, lam3, nFactorsInit)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% The main function implementing our algorithm
% Inputs:
%   - X: Matrix of predictors (n x p)
%   - Y: Matrix of responses (n x q)
%   - lam1: Regularization factor for ||A||_1
%   - lam2: Regularization factor for ||B||_1
%   - lam3: Regularization factor for ||A||^2_F
%   - nFactorsInit: Initial number of factors. Algorithm will start from
%   this number and reduce it until B has full row rank
% Outputs:
%   - D: Overall regression matrix (p x q). Equals A*B.
%   - A: Matrix deriving factors from inputs (p x m)
%   - B: Matrix of regression coefficients from factors to outputs (m x q)
%   - nFactors: Estimated number of factors
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


% Setting parameters
maxIter = 10000;
tol=1E-6;
nPreds = size(X,2);
nResps = size(Y,2);

for nFactors = nFactorsInit:-1:1
    A = rand(nPreds, nFactors);
    B = rand(nFactors,nResps);
    Aprev=A;
    Bprev=B;
    obj = 0.5*norm(Y-X*A*B,'fro')^2 + lam1*sum(abs(A(:))) + ...
        lam2*sum(abs(B(:)))+lam3*norm(A,'fro');
    t0=1;
    for iter=1:maxIter
        % --- updating B ---
        t = (1+sqrt(1+4*t0^2))/2;
        wB = (t0-1)/t;
        t0=t;
        LB = norm(A'*(X'*X)*A,'fro');
        Bhat = B + wB * (B-Bprev);
        Ghat = (A'*X'*(X*A*Bhat-Y));
        Bnew = wthresh(Bhat-Ghat/LB,'s', lam2/LB);
        obj_prev = obj;
        obj = 0.5*norm(Y-X*A*Bnew,'fro')^2 + lam1*sum(abs(A(:))) + ...
            lam2*sum(abs(Bnew(:)))+lam3*norm(A,'fro');
        Bprev=B;
        % --- if no descent, repeat with no extrapolation ---
        if obj>=obj_prev
            Ghat = (A'*X'*(X*A*B-Y));
            B = wthresh(B-Ghat/LB,'s', lam2/LB);
        else
            B=Bnew;
        end
        % --- updating A ---
        t = (1+sqrt(1+4*t0^2))/2;
        wA = (t0-1)/t;
        t0=t;
        LA = norm(X'*X,'fro')*norm(B*B','fro')+2*lam3;
        Ahat = A + wA * (A-Aprev);
        Ghat = (-X'*Y*B' + (X'*X)*Ahat*(B*B') + 2*lam3*Ahat);
        Anew = wthresh(Ahat-Ghat/LA,'s', lam1/LA);
        obj_prev = obj;
        obj = 0.5*norm(Y-X*Anew*B,'fro')^2 + lam1*sum(abs(Anew(:))) + ...
            lam2*sum(abs(B(:)))+lam3*norm(A,'fro');
        Aprev=A;
        % --- if no descent, repeat with no extrapolation ---
        if obj>=obj_prev
            Ghat = (-X'*Y*B' + (X'*X)*A*(B*B') + 2*lam3*A);
            A = wthresh(A-Ghat/LA,'s', lam1/LA);
        else
            A=Anew;
        end
        
        % --- stopping check ---
        obj = 0.5*norm(Y-X*A*B,'fro')^2 + lam1*sum(abs(A(:))) + ...
            lam2*sum(abs(B(:)))+lam3*norm(A,'fro');
        diff = abs(obj-obj_prev)/obj_prev;
        if diff<tol || isnan(diff)
            break;
        end
    end
    % --- checking the rank condition ---
    if ~isnan(diff) && ((rank(B) == nFactors) && (rank(A) == nFactors))
        D=A*B;
        return;
    end
end
D=A*B;
end
